#include "common/platform.h"
#include "common/cs_dbg.h"
#include "mgos_bitbang.h"
#include "mgos_gpio.h"
#include "mgos_system.h"
#include "mgos_sys_config.h"
#include "NeoPixel.h"

static int NeoPixel_pin = 0;
static int NeoPixel_numPixels = 0;
static uint8_t *NeoPixel_leds = NULL; // pointer to allocated memory
static enum NeoPixel_ColorOrder NeoPixel_colorOrder = NeoPixel_colorOrder_GRB;
static int brightness = 50;

static uint8_t adjustBrightness(uint8_t value) {
	return (value * brightness)/100;
}

int NeoPixel_getBrightness() { return brightness; }
void NeoPixel_setBrightness(int newBrightness) {
	if (newBrightness > 0 && newBrightness <= 100) {
			brightness = newBrightness;
	} else {
		LOG(LL_ERROR, ("invalid brightness value %d", newBrightness));
	}
}


// ## **`NeoPixel_clear()`**
// Clear in-memory values of the pixels.
void NeoPixel_clear() {
  if (NeoPixel_leds == NULL) {
    LOG(LL_ERROR, ("NeoPixel data space not allocated"));
    return;
  }
  for (int i=0; i<NeoPixel_numPixels * 3; ++i) {
    NeoPixel_leds[i] = 0;
  }
}

// ## **`NeoPixel_create(pin, numPixels, order)`**
// Create and return a NeoPixel strip object. Example:
// ```javascript
// let pin = 5, numPixels = 16, colorOrder = NeoPixel.GRB;
// let strip = NeoPixel.create(pin, numPixels, colorOrder);
// strip.setPixel(0 /* pixel */, 12, 34, 56);
// strip.show();
//
// strip.clear();
// strip.setPixel(1 /* pixel */, 12, 34, 56);
// strip.show();
// ```
void NeoPixel_create(uint8_t pin, uint8_t numPixels, enum NeoPixel_ColorOrder order) {
  mgos_gpio_init();
  NeoPixel_pin = pin;
  NeoPixel_numPixels = numPixels;
  NeoPixel_colorOrder = order;
  NeoPixel_leds = malloc(NeoPixel_numPixels * 3);
  // GPIO.set_mode(pin, GPIO.MODE_OUTPUT);
  mgos_gpio_set_mode(pin, MGOS_GPIO_MODE_OUTPUT);
  // GPIO.write(pin, 0);  // Keep in reset.
  mgos_gpio_write(pin, false);
  brightness = mgos_sys_config_get_led_brightness();
  LOG(LL_INFO, ("pin=%d, numPixels=%d, colorOrder=%d, brightness=%d", pin, numPixels, order, brightness));
  NeoPixel_clear();
}

void NeoPixel_release() {
  if (NeoPixel_leds == NULL) {
    LOG(LL_ERROR, ("NeoPixel data space not allocated"));
    return;
  }
  free(NeoPixel_leds);
  NeoPixel_leds = NULL;
  NeoPixel_numPixels = 0;
}
// ## **`NeoPixel_set(i, r, g, b)`**
// Set i-th's pixel's RGB value.
// Note that this only affects in-memory value of the pixel.
void NeoPixel_set(int pixel, int red, int green, int blue) {
  uint8_t v0, v1, v2;
  if (NeoPixel_leds == NULL) {
    LOG(LL_ERROR, ("NeoPixel data space not allocated"));
    return;
  }
  if (pixel < 0 || pixel >= NeoPixel_numPixels) {
    LOG(LL_ERROR, ("Invalid pixel number %d", pixel));
    return;
  }

  if (NeoPixel_colorOrder == NeoPixel_colorOrder_RGB) {
    v0 = red; v1 = green; v2 = blue;
  } else if (NeoPixel_colorOrder == NeoPixel_colorOrder_GRB) {
    v0 = green; v1 = red; v2 = blue;
  } else if (NeoPixel_colorOrder == NeoPixel_colorOrder_BGR) {
    v0 = blue; v1 = green; v2 = red;
  } else return;
  NeoPixel_leds[3*pixel] = adjustBrightness(v0);
  NeoPixel_leds[3*pixel+1] = adjustBrightness(v1);
  NeoPixel_leds[3*pixel+2] = adjustBrightness(v2);
}

// ## **`NeoPixel_show()`**
// Output values of the pixels.
void NeoPixel_show() {
  if (NeoPixel_leds == NULL) {
    LOG(LL_ERROR, ("NeoPixel data space not allocated"));
    return;
  }

  // GPIO.write(this.pin, 0);
  mgos_gpio_write(NeoPixel_pin, false);
  // Sys.usleep(60);
  mgos_usleep(60);
  // BitBang.write(this.pin, BitBang.DELAY_100NSEC, 3, 8, 8, 6, this.data, this.len);
  mgos_bitbang_write_bits(NeoPixel_pin, MGOS_DELAY_100NSEC, 3, 8, 8, 6, NeoPixel_leds, 3 * NeoPixel_numPixels);
  // GPIO.write(this.pin, 0);
  mgos_gpio_write(NeoPixel_pin, false);
  // Sys.usleep(60);
  mgos_usleep(60);
  // GPIO.write(this.pin, 1);
  mgos_gpio_write(NeoPixel_pin, true);
}
